﻿using System;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Utilities;
using Microsoft.SharePoint.Taxonomy;

namespace WingtipToysTermStoreManager.TermStoreUpdatesReceiver
{
    /// <summary>
    /// List Item Events
    /// </summary>
    public class TermStoreUpdatesReceiver : SPItemEventReceiver
    {
/// <summary>
/// An item was added.
/// </summary>
public override void ItemAdded(SPItemEventProperties properties)
{
    if (properties.List.Title.Contains("Term Store Updates"))
    {
        SPListItem itemToProcess = properties.ListItem;
        Guid siteId = properties.SiteId;

        ProcessChanges(itemToProcess, siteId);

        base.ItemAdded(properties);
    }
}

/// <summary>
/// An item was updated.
/// </summary>
public override void ItemUpdated(SPItemEventProperties properties)
{
    if (properties.List.Title.Contains("Term Store Updates"))
    {
        SPListItem itemToProcess = properties.ListItem;
        Guid siteId = properties.SiteId;

        ProcessChanges(itemToProcess, siteId);

        base.ItemUpdated(properties);
    }
}

/// <summary>
/// Process list item changes in the term store
/// </summary>
/// <param name="itemToProcess"></param>
/// <param name="siteId"></param>
public void ProcessChanges(SPListItem itemToProcess, Guid siteId)
{
    if (!Convert.ToBoolean(itemToProcess["Processed"]))
    {
        using (SPSite site = new SPSite(siteId))
        {
            TaxonomySession taxonomySession = new TaxonomySession(site);
            if (taxonomySession.TermStores.Count > 0)
            {
                TermStore termStore = taxonomySession.TermStores[0];

                // Check to make sure that the termstore is online
                if (termStore.IsOnline)
                {
                    Group termGroup;
                    TermSet termSet;
                    string action = itemToProcess["Action"].ToString();
                    bool itemUpdated = false;

                    // TermGroup
                    try
                    {
                        // Get the term group
                        termGroup = termStore.Groups[itemToProcess["TermGroup"].ToString()];
                    }
                    catch
                    {
                        termGroup = termStore.CreateGroup(itemToProcess["TermGroup"].ToString());
                    }

                    // TermSet
                    try
                    {
                        termSet = termGroup.TermSets[itemToProcess["TermSet"].ToString()];
                    }
                    catch
                    {
                        termSet = termGroup.CreateTermSet(itemToProcess["TermSet"].ToString());
                    }

                    string termValue = itemToProcess["Term"].ToString();
                    Guid termGuid;

                    switch (action)
                    {
                        case "Create":
                            if (itemToProcess["TermID"] != null)
                            {
                                termGuid = new Guid(itemToProcess["TermID"].ToString());
                            }
                            else
                            {
                                termGuid = Guid.NewGuid();
                            }

                            Term newTerm = termSet.CreateTerm(termValue, 1033, termGuid);
                            itemToProcess["TermID"] = termGuid;
                            itemUpdated = true;

                            break;
                        case "Update":
                            if (itemToProcess["TermID"] != null)
                            {
                                termGuid = new Guid(itemToProcess["TermID"].ToString());
                                Term updatedTerm = termSet.GetTerm(termGuid);
                                Label updatedLabel = updatedTerm.Labels[0];
                                updatedLabel.Value = termValue;
                                itemUpdated = true;
                            }
                            break;
                        case "Deprecate":
                            if (itemToProcess["TermID"] != null)
                            {
                                termGuid = new Guid(itemToProcess["TermID"].ToString());
                                Term deprecatedTerm = termSet.GetTerm(termGuid);
                                deprecatedTerm.Deprecate(true);
                                itemUpdated = true;
                            }
                            break;
                        case "Restore":
                            if (itemToProcess["TermID"] != null)
                            {
                                termGuid = new Guid(itemToProcess["TermID"].ToString());
                                Term deprecatedTerm = termSet.GetTerm(termGuid);
                                deprecatedTerm.Deprecate(false);
                                itemUpdated = true;
                            }
                            break;
                        case "Delete":
                            if (itemToProcess["TermID"] != null)
                            {
                                termGuid = new Guid(itemToProcess["TermID"].ToString());
                                Term deletedTerm = termSet.GetTerm(termGuid);
                                deletedTerm.Delete();
                                itemUpdated = true;
                            }
                            break;
                        default:
                            break;
                    }

                    termStore.CommitAll();
                    if (itemUpdated)
                    {
                        itemToProcess["Processed"] = true;

                        itemToProcess.Update();
                    }

                }
            }
        }
    }
}
    }
}